@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 
 #
 # Copyright 2015, Daehwan Kim <infphilo@gmail.com>
@@ -23,6 +23,7 @@
 import sys, subprocess
 import re
 from argparse import ArgumentParser, FileType
+from functools import cmp_to_key
 
 
 """
@@ -80,8 +81,8 @@ def compare_vars(a, b):
 
     # daehwan - for debugging purposes
     if a_chr != b_chr:
-        print a
-        print b
+        print(a)
+        print(b)
     
     assert a_chr == b_chr
     if a_pos != b_pos:
@@ -129,7 +130,7 @@ def generate_haplotypes(snp_file,
     assert len(vars) > 0
 
     # Sort variants and remove redundant variants
-    vars = sorted(vars, cmp=compare_vars)
+    vars = sorted(vars, key=cmp_to_key(compare_vars))
     tmp_vars = []
     v = 0
     while v < len(vars):
@@ -223,8 +224,8 @@ def generate_haplotypes(snp_file,
         else:
             assert type == 'I'
             type = "insertion"
-        print >> snp_file, "%s\t%s\t%s\t%s\t%s" % \
-            (varID, type, chr, pos, data)
+        print("%s\t%s\t%s\t%s\t%s" % (varID, type, chr, pos, data),
+                file=snp_file)
 
     # genotypes_list looks like
     #    Var0: 0
@@ -270,7 +271,7 @@ def generate_haplotypes(snp_file,
                     split_haplotypes.add('#'.join(haplotype[prev_s:s]))
         return split_haplotypes
 
-    haplotypes2 = split_haplotypes(haplotypes)
+    haplotypes2 = sorted(list(split_haplotypes(haplotypes)))
 
     def cmp_haplotype(a, b):
         a = a.split('#')
@@ -288,8 +289,8 @@ def generate_haplotypes(snp_file,
         if a_begin != b_begin:
             return a_begin - b_begin
         return a_end - b_end
-    
-    haplotypes = sorted(list(haplotypes2), cmp=cmp_haplotype)
+
+    haplotypes = sorted(list(haplotypes2), key=cmp_to_key(cmp_haplotype))
 
     # Write haplotypes
     for h_i in range(len(haplotypes)):
@@ -317,8 +318,8 @@ def generate_haplotypes(snp_file,
         for id in h:
             var_dic = vars[int(id)][4]
             h_add.append(var_dic["id2"])
-        print >> haplotype_file, "ht%d\t%s\t%d\t%d\t%s" % \
-            (num_haplotypes, chr, h_new_begin, h_end, ','.join(h_add))
+        print("ht%d\t%s\t%d\t%d\t%s" % (num_haplotypes, chr, h_new_begin, h_end, ','.join(h_add)),
+                file=haplotype_file)
         num_haplotypes += 1
 
     return num_haplotypes
@@ -352,6 +353,7 @@ def main(genome_file,
     else:
         snp_cmd = ["cat", snp_fname]
     snp_proc = subprocess.Popen(snp_cmd,
+                                universal_newlines=True,
                                 stdout=subprocess.PIPE,
                                 stderr=open("/dev/null", 'w'))
     ids_seen = set()
@@ -447,10 +449,10 @@ def main(genome_file,
                 if testset:
                     ref_seq = chr_seq[start-50:start+50]
                     alt_seq = chr_seq[start-50:start] + allele + chr_seq[start+1:start+50]
-                    print >> ref_testset_file, ">%s_single_%d" % (rs_id, start - 50)
-                    print >> ref_testset_file, ref_seq
-                    print >> alt_testset_file, ">%s_single_%d_%s" % (rs_id, start - 50, ref_seq)
-                    print >> alt_testset_file, alt_seq
+                    print(">%s_single_%d" % (rs_id, start - 50), file=ref_testset_file)
+                    print(ref_seq, file=ref_testset_file)
+                    print(">%s_single_%d_%s" % (rs_id, start - 50, ref_seq), file=alt_testset_file)
+                    print(alt_seq, file=alt_testset_file)
                 
         elif classType == "deletion":
             if start > 0:
@@ -475,10 +477,10 @@ def main(genome_file,
             if testset and delLen > 0 and delLen <= 10:
                 ref_seq = chr_seq[start-50:start+50]
                 alt_seq = chr_seq[start-50:start] + chr_seq[start+delLen:start+50+delLen]
-                print >> ref_testset_file, ">%s_deletion_%d" % (rs_id, start - 50)
-                print >> ref_testset_file, ref_seq
-                print >> alt_testset_file, ">%s_deletion_%d_%s" % (rs_id, start - 50, ref_seq)
-                print >> alt_testset_file, alt_seq
+                print(">%s_deletion_%d" % (rs_id, start - 50), file=ref_testset_file)
+                print(ref_seq, file=ref_testset_file)
+                print(">%s_deletion_%d_%s" % (rs_id, start - 50, ref_seq), file=alt_testset_file)
+                print(alt_seq, file=alt_testset_file)
         else:
             assert classType == "insertion"
             if start > 0:
@@ -497,10 +499,10 @@ def main(genome_file,
                     if testset and insLen > 0 and insLen <= 10:
                         ref_seq = chr_seq[start-50:start+50]
                         alt_seq = chr_seq[start-50:start] + allele + chr_seq[start:start+50-insLen]
-                        print >> ref_testset_file, ">%s_insertion_%d" % (rs_id, start - 50)
-                        print >> ref_testset_file, ref_seq
-                        print >> alt_testset_file, ">%s_insertion_%d_%s" % (rs_id, start - 50, ref_seq)
-                        print >> alt_testset_file, alt_seq
+                        print(">%s_insertion_%d" % (rs_id, start - 50), file=ref_testset_file)
+                        print(ref_seq, file=ref_testset_file)
+                        print(">%s_insertion_%d_%s" % (rs_id, start - 50, ref_seq), file=alt_testset_file)
+                        print(alt_seq, file=alt_testset_file)
 
         if curr_right < end:
             curr_right = end
